/******************************************************************************
 * notes.c
 *-----------------------------------------------------------------------------
 * Copyright (C) 2014-2015 Osman Celimli
 * For conditions of distribution and use, see copyright notice in hscc.c
 ******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
/******************************************************************************
 * Text Tables
 ******************************************************************************/
char ERR_BadNote[]= "Unable to parse note : ";

/******************************************************************************
 * Note Tables
 ******************************************************************************/
/* Note Textual Equivalents
 -----------------------------*/
char aszNote_BaseText[][3]={"cb",		   "bs","c.","cs","db",
							"d.","ds","eb","e.","fb","es","f.",
							"fs","gb","g.","gs","ab","a.",
							"as","bb","b."};
/* Base Offset from Octave
 -----------------------------*/
int aiNote_BaseOffset[]=   {11,				0,0,1,1,
							2,3,3,			4,4,5,5,
							6,6,7,			8,8,9,
							10,10,11};
/* Inter-Octave Adjustment
 -----------------------------*/
int aiNote_OctOffset[]=	  {-1,				1,0,0,0,
							0,0,0,			0,0,0,0,
							0,0,0,			0,0,0,
							0,0,0,0};

#define NOTE_REFLEN (sizeof(aszNote_BaseText) / sizeof(aszNote_BaseText[0]))

/******************************************************************************
 * int note_getFreq(char *pszNote, double *pdFreq)
 * Attempt to parse the note string supplied in szNote, storing its equivalent
 * frequency in A4,440Hz tuning in *pdFreq.
 *
 * Uses a three characer format arranged as : nXo
 * n = Note (a,g,f,etc)
 * X = Flat (b) or Sharp (s)
 * o = Octave (0+)
 *
 * Returns a nonzero value if the parsing failed.
 ******************************************************************************/
int note_getFreq(char *pszNote, double *pdFreq)
{
	char szNoteBuf[3],*pszOctave;
	unsigned int uiNoteLen;

	int iNote,iOctave;
	int iBaseOctave,iOctaveOffset;

	double dNote,dFreq;
	unsigned int uiX;


	if((NULL == pszNote) || (NULL == pdFreq))
		return -1;
	if((uiNoteLen = strlen(pszNote)) < 3)
	{
		fprintf(stderr, "%s%s\n", ERR_BadNote, pszNote);
		return -1;
	}

	szNoteBuf[0] = pszNote[0];
	szNoteBuf[1] = pszNote[1];
	szNoteBuf[2] = 0;
	pszOctave = pszNote + 2;
	if(1 != sscanf(pszOctave, "%d", &iBaseOctave))
	{
		fprintf(stderr, "%s%s\n", ERR_BadNote, pszNote);
		return -1;
	}

	for(uiX = 0; uiX < NOTE_REFLEN; uiX++)
	{
		if(0 == strcmp(aszNote_BaseText[uiX], szNoteBuf))
			break;
	}
	if(NOTE_REFLEN == uiX)
	{
		fprintf(stderr, "%s%s\n", ERR_BadNote, pszNote);
		return -1;
	}

	iNote = aiNote_BaseOffset[uiX];
	iOctaveOffset = aiNote_OctOffset[uiX];
	iOctave = iBaseOctave + iOctaveOffset;

	/**
	 * Frequency is calculated relative to A4,440 using:
	 *  f= 2^(((12 * Octave) + Note) / 12) * 440
	 */
	dNote = ((12 * iOctave) + iNote) - ((12 * 4) + 9);
	dFreq = fabs(pow(2.0, (dNote / 12.0)) * 440.0);
	
	*pdFreq = dFreq;
	return 0;
}
