/******************************************************************************
 * sfx.h
 *-----------------------------------------------------------------------------
 * Copyright (C) 2014-2015 Osman Celimli
 * For conditions of distribution and use, see copyright notice in hscc.c
 ******************************************************************************/
#ifndef SFX
#define SFX

#define SFX_CHANNELS					HU_CHANNELS

/* Sound Effect Headers
 -----------------------------*/
#define SFX_MAGICWORD					"SFX"
#define SFX_PACK_OFFSET_MAGIC			0
#define SFX_PACK_OFFSET_COUNT			3
#define SFX_PACK_OFFSET_DIR				4
#define SFX_PACK_DIR_LEN				4
#define SFX_PACK_LEN					4

#define SFX_HEADER_OFFSET_PRI			0
#define SFX_HEADER_OFFSET_COUNT			1
#define SFX_HEADER_OFFSET_DESCS			2
#define SFX_HEADER_LEN					2

#define SFX_NOTEOFF_LEN					4

#define SFX_CHANNEL_OFFSET_REQS			0
#define SFX_CHANNEL_OFFSET_ADDR			1
#define SFX_CHANNEL_OFFSET_ADDRL		SFX_CHANNEL_OFFSET_ADDR + 0
#define SFX_CHANNEL_OFFSET_ADDRH		SFX_CHANNEL_OFFSET_ADDR + 1
#define SFX_CHANNEL_OFFSET_ADDRB		SFX_CHANNEL_OFFSET_ADDR + 2
#define SFX_CHANNEL_OFFSET_ADDRM		SFX_CHANNEL_OFFSET_ADDR + 3
#define SFX_CHANNEL_DESC_LEN			5

/* Sound Effect Script Commands
 -----------------------------*/
#define SFX_COMMAND_END					0
#define SFX_COMMAND_DELAY				1

#define SFX_COMMAND_LOOP_START			2
#define SFX_COMMAND_LOOP_END			3

#define SFX_COMMAND_WAVEFORM			4
#define SFX_COMMAND_SAMPLE				5
#define SFX_COMMAND_LFSR				6
#define SFX_COMMAND_FREQUENCY			7
#define SFX_COMMAND_VOLUME				8

/* Buffer Setup
 -----------------------------*/
#define SFX_PARSER_LINEBUF_LEN			512
#define SFX_PARSER_SFXBUF_LEN			64
#define SFX_PARSER_STREAM_LEN			1024

/* Value Ranges
 -----------------------------*/
#define SFX_PRIORITY_MIN				0
#define SFX_PRIORITY_MAX				255
#define SFX_WAIT_MIN					0
#define SFX_WAIT_MAX					255
#define SFX_LOOP_MIN					-128
#define SFX_LOOP_MAX					127
#define SFX_SAMPLE_MIN					0
#define SFX_SAMPLE_MAX					255


/* Channel and Block Containers
 -----------------------------*/
typedef struct SFX_Channel
{
	int iAmClosed,iUseNoise;

	char *pacStream;
	unsigned int uiScriptLen,uiBufferLen;
} SFX_Channel;

typedef struct SFX_Block
{
	char *pszName;

	int iAmInstrument;
	unsigned int uiNoteOff;
	double dTuning;

	unsigned int uiPriority,uiChannels;
	SFX_Channel aChannelData[SFX_CHANNELS];
} SFX_Block;

/******************************************************************************
 * Function Defines
 ******************************************************************************/
int sfx_bodySASS(SFX_Block *pSFX, char *pszMainTok, int *piBodyOpen);
int sfx_parseSASS(FILE *pSASS, char *pszFileName, int iDoSFX);

int sfx_buildSFX(char **ppacSFX, unsigned int *puiBinLen);
int sfx_writeDefs(char *pszSrcName);

int sfx_getInstInfo(unsigned int uiInstrument, int *piUseNoise,
					double *pdTuning);
int sfx_buildInst(char **ppacInst, unsigned int *puiBinLen,
				  unsigned int *puiNumInst);

int sfx_addByte(SFX_Channel *pChannel, char cByte);
int sfx_addSFX(SFX_Block *pSFX);
int sfx_find(char *pszName);
void sfx_closeTable(void);
#endif
