;****************************************************************
;	 HuSound Auditioning Environment for MCGenjins		*
;****************************************************************
; Version 1.3cz, November 10th, 2015
; Copyright (C) 2014-2015 Osman Celimli
;
; This software is provided 'as-is', without any express or implied
; warranty.  In no event will the authors be held liable for any damages
; arising from the use of this software.
;
; Permission is granted to anyone to use this software for any purpose,
; including commercial applications, and to alter it and redistribute it
; freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must not
;    claim that you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation would be
;    appreciated but is not required.
; 2. Altered source versions must be plainly marked as such, and must not be
;    misrepresented as being the original software.
; 3. This notice may not be removed or altered from any source distribution.
;----------------------------------------------------------------
; Mapping Setup
;----------------------------------------------------------------
.include "./EQU_MAP.asm"

; Hardware Equates
;----------------------------------------------------------------
.include "./EQU_PCE.asm"
.include "./EQU_VDC.asm"
.include "./EQU_VCE.asm"
.include "./EQU_MCG.asm"

; Library Configs
;----------------------------------------------------------------
.define HUSOUND_CHANNELS	6
.define HUSOUND_STACKLEN	4
.define HUSOUND_REQLEN		8

; Variable Defines
;----------------------------------------------------------------
; ZeroPage ($2000-$20FF)
;----------
.printt "---- ZEROPAGE ----\n"
.include "./ZP_SYS.asm"
.include "./../HuSound/ZP_HuSound.asm"
.printt "\n"

; Base RAM ($2200-$3FFF)
;----------
.printt "---- BASE RAM ($2200-$3FFF) ----\n"
.include "./VAR_VID.asm"
.include "./../HuSound/VAR_HuSound.asm"
.include "./VAR_USER.asm"


;****************************************************************
; System Libraries and Initialization Routines			|
;----------------------------------------------------------------
.bank BANK_SYSTEM_A slot SLOT_SYSTEM_A
.orga ADDR_SYSTEM_A

; Usefuls
;-------------------------------
.include "./SUB/System/NIBBLES.asm"
.include "./SUB/Engine/RUNSEL.asm"

; HuSound
;-------------------------------
.include "./../HuSound/HuSound.asm"
.include "./../HuSound/HuSFX.asm"
.include "./../HuSound/HuMusic.asm"
.include "./../HuSound/HuSample.asm"

; Hardware Drivers
;-------------------------------
.include "./SUB/System/PCE_MCG.asm"
.include "./SUB/System/PCE_JOY.asm"
.include "./SUB/System/PCE_VDC.asm"
.include "./SUB/System/PCE_VCE.asm"
.include "./SUB/System/PCE_VBL.asm"
.include "./SUB/System/PCE_INIT.asm"	; <- Must be at the end of Bank 0

; Reset Vectors
;---------------
.orga $fff6
CPUVec_IRQ2:	.dw IRQ_Dummy
CPUVec_IRQ1:	.dw IRQ_VDC
CPUVec_TIMER:	.dw IRQ_HuSample
CPUVec_NMI:	.dw IRQ_Dummy
CPUVec_RESET:	.dw PCE_Reset

.bank BANK_SYSTEM_B slot SLOT_SYSTEM_B
.orga ADDR_SYSTEM_B

; Main main maine moose?
;-------------------------------
.include "./SUB/Engine/MAIN.asm"

; Text / Font Drawing Routines
;-------------------------------
.include "./SUB/Engine/FWF_DBG.asm"


;****************************************************************
; Common Resources						|
;----------------------------------------------------------------
; Fonts
;-------------------------------
.bank OBJBANK_FONTS
.orga ADDR_OBJ
.include "./DAT_Fonts.asm"


;****************************************************************
; Glue Screens							|
;----------------------------------------------------------------
; Sound Test
;-------------------------------
.bank OBJBANK_SOUNDTEST
.orga ADDR_OBJ
.include "./SUB/SoundTest/VAR_Audition.asm"
.include "./SUB/SoundTest/USR_Audition.asm"
.include "./SUB/SoundTest/DrawState.asm"


;****************************************************************
; Sound Data							|
;----------------------------------------------------------------
; Sound Effects Package
;-------------------------------
.fopen "./SASS/SFXPAK.bin" fSFX
.fsize fSFX sSFX
.define bSFX 0
.repeat (sSFX / OBJ_SIZE)
	.bank (OBJBANK_SFXDATA + bSFX)
	.orga ADDR_OBJ
	.repeat OBJ_SIZE
		.fread fSFX d
		.db d
	.endr
	.redefine bSFX (bSFX + 1)
.endr
.bank (OBJBANK_SFXDATA + bSFX)
.orga ADDR_OBJ
.repeat (sSFX # OBJ_SIZE)
	.fread fSFX d
	.db d
.endr
.fclose fSFX

; Track Package
;-------------------------------
.fopen "./SASS/MUSPAK.bin" fMUS
.fsize fMUS sMUS
.define bMUS 0
.repeat (sMUS / OBJ_SIZE)
	.bank (OBJBANK_MUSDATA + bMUS)
	.orga ADDR_OBJ
	.repeat OBJ_SIZE
		.fread fMUS d
		.db d
	.endr
	.redefine bMUS (bMUS + 1)
.endr
.bank (OBJBANK_MUSDATA + bMUS)
.orga ADDR_OBJ
.repeat (sMUS # OBJ_SIZE)
	.fread fMUS d
	.db d
.endr
.fclose fMUS

; PCM Sample Pack
;-------------------------------
.fopen "./SASS/PCMPAK.bin" fPCM
.fsize fPCM sPCM
.define bPCM 0
.repeat (sPCM / OBJ_SIZE)
	.bank (OBJBANK_PCMDATA + bPCM)
	.orga ADDR_OBJ
	.repeat OBJ_SIZE
		.fread fPCM d
		.db d
	.endr
	.redefine bPCM (bPCM + 1)
.endr
.bank (OBJBANK_PCMDATA + bPCM)
.orga ADDR_OBJ
.repeat (sPCM # OBJ_SIZE)
	.fread fPCM d
	.db d
.endr
.fclose fPCM
