;****************************************************************
;	      Anything, Everything, All the Things		*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
;

;****************************************************************
; Start:							*
;	Additional initializations before the main loop. IRQs	*
; should still be off before entering this startup phase.	*
;****************************************************************
.section "Start" force
Start:

; HuSound
;---------
	LDA #(BANK_SFXDATA / MCBANK_PAGES)
	STA HuSFX_PackBase + MCMAP
	LDA #((BANK_SFXDATA # MCBANK_PAGES) + BANK_ROM)
	STA HuSFX_PackBase + BANK
	LDA #(>ADDR_OBJ)
	STA HuSFX_PackBase + HI

	LDA #(BANK_MUSDATA / MCBANK_PAGES)
	STA HuMusic_PackBase + MCMAP
	LDA #((BANK_MUSDATA # MCBANK_PAGES) + BANK_ROM)
	STA HuMusic_PackBase + BANK
	LDA #(>ADDR_OBJ)
	STA HuMusic_PackBase + HI

	LDA #(BANK_PCMDATA / MCBANK_PAGES)
	STA HuSample_DirBase + MCMAP
	LDA #((BANK_PCMDATA # MCBANK_PAGES) + BANK_ROM)
	STA HuSample_DirBase + BANK
	LDA #(>ADDR_OBJ)
	STA HuSample_DirBase + HI

	jsr HuSound_Init	; All handlers are ready, so...
	cli			; IRQs back on!

; System Palettes and Font
;--------------------------
	LDA #<_PAL_SYSTEM
	STA PAL_Src + LO
	LDA #>_PAL_SYSTEM
	STA PAL_Src + HI
	STZ PAL_Src + BANK
	STZ PAL_Src + MCMAP
	STZ PAL_Dest
	jsr VCE_LoadPal

	LDA #<_COPY_SYSFONT
	STA VDAT_BlockSrc + LO
	LDA #>_COPY_SYSFONT
	STA VDAT_BlockSrc + HI
	jsr VDC_MakeVREQ_B

; Clear Maps
;------------
	LDA #<_CLEAR_MAPS
	STA VDAT_BlockSrc + LO
	LDA #>_CLEAR_MAPS
	STA VDAT_BlockSrc + HI
	jsr VDC_MakeVREQ_B

; Setup First Action
;--------------------
	LDA #$FF
	STA GLO_ForceInit
	STZ GLO_isGameMode

	LDA #(BANK_SOUNDTEST / MCBANK_PAGES)
	STA GLO_UserRoutine + MCMAP
	LDA #((BANK_SOUNDTEST # MCBANK_PAGES) + BANK_ROM)
	STA GLO_UserRoutine + BANK
	LDA #<USR_Audition
	STA GLO_UserRoutine + LO
	LDA #>USR_Audition
	STA GLO_UserRoutine + HI

; First VBL
;-----------
	INC FrameDone
_waitVBL			; The first VBL is a special case, we'll
	LDA FrameDone		; wait for it to trigger and pass so all of
	bne _waitVBL		; the VREQs made above can be serviced.
	jmp Main

_CLEAR_MAPS	.db (VDC_TRANSFER_I1 | VDC_TRANSFER_FILL)	; TYPE
		.dw VCEL_SYSFONT + 0				; SOURCE LO,HI
		.db 0,0						; SOURCE BANK,MAP
		.dw VMEM_MAINSCR				; DEST LO,HI
		.dw (VMAP_SIZE * 3)				; SIZE

_COPY_SYSFONT	.db (VDC_TRANSFER_I1 | VDC_TRANSFER_1BBG4)	; TYPE
		.dw FWF8_SYS					; SOURCE LO,HI
		.db BANK_FONTS,0				; SOURCE BANK,MAP
		.dw VMEM_SYSFONT				; DEST LO,HI
		.dw (FWF8_SYS_END - FWF8_SYS)			; SIZE

_PAL_SYSTEM	.dw $0000,$FF01,$FF01,$FF01,$FF01,$FF01,$FF01,$FF01
		.dw $FF01,$FF01,$FF01,$FF01,$FF01,$FF01,$FF01,$FF01
.ends


;****************************************************************
; Main:								*
;****************************************************************
.section "Main" force
Main:

; Start of Frame
;----------------
	INC FrameCounter
	jsr ReadJoy
	jsr HandleReset
	jsr HandlePause


; Game / User Mode Select
;-------------------------
	LDA GLO_isGameMode
	bne Main_SystemCalls
_userRoutine
	LDA GLO_UserRoutine + BANK
	STA MCG_Bank
	LDA GLO_UserRoutine + MCMAP
	STA MCG_Map
	MCG_Sel			; ~! MCGENJIN OBJ SELECT !~

	LDA GLO_UserRoutine + LO
	STA GLO_JumpDest + LO
	LDA GLO_UserRoutine + HI
	STA GLO_JumpDest + HI
	jsr GLO_VarJump + ADDR_RAM

	MCG_Res			; ~! MCGENJIN OBJ RESTORE !~
	jmp Main_EOF


; System Update Calls
;---------------------
Main_SystemCalls:


; End of Frame
;--------------
Main_EOF:
	INC FrameDone		; We're done with frame processing, so...
_waitVBL
	.db $CB			; Take a nap... (WAI? Says Johnny)
	LDA FrameDone		; ...until VBL has come along and done its thing.
	bne _waitVBL
	jmp Main
.ends
