;****************************************************************
;		    System ZeroPage Defines			*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
; Note- I'm actually defining this in the $00-$FF range so that
; WLA-DX will use one byte addresses without the ">" operator.

.enum SYSTEM_ZP_BASE
;****************************************************************
;		System Management and Temporaries		*
;****************************************************************
SysRegion		ds 1	; <- This must be the first ZP entry

; Frame Management
;------------------
FrameCounter		ds 1
FrameDone		ds 1
VBL_Busy		ds 1

; Gamestick Joyport
;-------------------
JOY_Current		ds 1
JOY_Last		ds 1
JOY_Pressed		ds 1
JOY_Released		ds 1

; General Use Pointers and Temporaries
;--------------------------------------
GLO_PointerA		ds 2
GLO_PointerB		ds 2
GLO_PointerC		ds 2
GLO_PointerD		ds 2

GLO_TempA		ds 1
GLO_TempB		ds 1
GLO_TempC		ds 1
GLO_TempD		ds 1

GLO_VarJump		ds 1	; Must be set to $4C
GLO_JumpDest		ds 2

; MCGenjin Mapper
;-----------------
MCG_Bank		ds 1	; Target Bank and Map for OBJ Slot
MCG_Map			ds 1	; selections.
MCG_LastMap		ds 1	; Backup of last write to MCGenjin

MCG_Base		ds 4	; Lo, Hi, Bank, MCGenjin Map
MCG_Offset		ds 4	; For use with address resolution functions, in-frame
MCG_Target		ds 4	; only. IRQs should use inlined address resolution.

;****************************************************************
;			  Game State				*
;****************************************************************
GLO_ForceInit		ds 1	; ($FF = Init on Next Frame, $00 = No Action)
GLO_isGameMode		ds 1	; ($FF = Use Engine / Scripts, $00 = Call User Routine)
GLO_UserRoutine		ds 4	; Lo, Hi, Bank, MCGenjin Map

; Pause?
;--------
GLO_isPaused		ds 1
GLO_isPausable		ds 1


;****************************************************************
;		      High-Level Display			*
;****************************************************************
; Font Rendering and Text
;-------------------------
DBG_TextSrc		ds 2
DBG_TextDest		ds 2


;****************************************************************
;		    VDC and VCE Interfacing			*
;****************************************************************
VBL_VarJump		ds 1	; Must be set to $4C
VBL_JumpDest		ds 2
VBL_PointerA		ds 2	; VBL-only pointers
VBL_PointerB		ds 2
HBL_PointerA		ds 2	; HBL-only pointers
HBL_PointerB		ds 2

; VDC Register Mirrors
;----------------------
VDCM_CR			ds 2	; These are copied to their respective
VDCM_RCR		ds 2	; VDC equivalents at the end of every
VDCM_BXR		ds 2	; frame.
VDCM_BYR		ds 2
VDCM_MWR		ds 2

; VCE Color Mirrors / Ringbuffer
;--------------------------------
VCE_BGColor		ds 2
VCE_CopySrc		ds 2
VCE_TxRing_I		ds 1
VCE_TxRing_O		ds 1

; Palette Adjustment Parameters
;-------------------------------
PAL_Src			ds 4	; Lo, Hi, Bank, MCGenjin Map
PAL_Dest		ds 2
PAL_AdjRed		ds 1
PAL_AdjBlue		ds 1
PAL_AdjGreen		ds 1
PAL_AdjInt		ds 1

; VDC Sprites & SAT
;-------------------
VDC_CurSAT		ds 1

; VDC Transfer Ringbuffer
;-------------------------
VDC_CopySrc		ds 2
VDC_TxRing_I		ds 1
VDC_TxRing_O		ds 1

; VRAM Transfer Request Parameters
;----------------------------------
VDAT_BlockSrc		ds 2	; <- For adding a fixed (in ROM) transfer block
VDAT_Type		ds 1	; <- To make your own transfer block
VDAT_Src		ds 4
VDAT_Dest		ds 2
VDAT_Size		ds 2


;****************************************************************
;			   ZeroPage End				*
;****************************************************************
SYSTEM_ZP_END		ds 1
.ende
.printt "System : $"
.printv HEX SYSTEM_ZP_BASE
.printt " to $"
.printv HEX SYSTEM_ZP_END - 1
.printt ", "
.printv DEC SYSTEM_ZP_END - SYSTEM_ZP_BASE
.printt " bytes\n"
